﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.UI;
using Telerik.Web.UI;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx;
using Telerik.Windows.Documents.Spreadsheet.Model;

public partial class _RadGrid : System.Web.UI.Page
{
    static GridItemType[] supportedItemTypes = new GridItemType[] 
                { 
                    GridItemType.Header, 
                    GridItemType.AlternatingItem, 
                    GridItemType.Item 
                };

    protected void RadGrid1_ItemCommand(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == RadGrid.ExportToExcelCommandName)
        {
            this.PreRender += (s, a) => { ExportToXlsx(); };
            e.Canceled = true;
        }
        else if (e.CommandName == RadGrid.ExportToWordCommandName)
        {
            this.PreRender += (s, a) => { ExportToDocx(); };
            e.Canceled = true;
        }
    }

    private void ExportToDocx()
    {
        string fileName = "RadGrid.docx";
        string filePath = Server.MapPath(fileName);

        IgnorePaging();

        RadFlowDocument flowDoc = new RadFlowDocument();
        Section section = flowDoc.Sections.AddSection();
        Table mainTable = section.Blocks.AddTable();

        GridItem item;
        TableRow row;

        var gridItems = RadGrid1.MasterTableView.GetItems(supportedItemTypes);

        for (int itemNum = 0; itemNum < gridItems.Length; itemNum++)
        {
            row = mainTable.Rows.AddTableRow();
            item = gridItems[itemNum];
            for (int cellNum = 0; cellNum < item.Cells.Count; cellNum++)
            {
                TableCell td = row.Cells.AddTableCell();
                Paragraph p = td.Blocks.AddParagraph();
                var currentGridCell = item.Cells[cellNum];
                p.Inlines.AddRun(GetCellText(currentGridCell.Text));

                if (!currentGridCell.Visible)
                {
                    td.PreferredWidth = new TableWidthUnit(0d);
                }
            }
        }

        GenerateFile(flowDoc, fileName);
    }

    private void ExportToXlsx()
    {
        string fileName = "RadGrid.xlsx";
        string filePath = Server.MapPath(fileName);

        IgnorePaging();

        Workbook workbook = new Workbook();
        Worksheet worksheet = workbook.Worksheets.Add();
        GridItem item;

        var gridColumns = RadGrid1.MasterTableView.RenderColumns;
        var gridItems = RadGrid1.MasterTableView.GetItems(supportedItemTypes);

        for (int itemNum = 0; itemNum < gridItems.Length; itemNum++)
        {
            item = gridItems[itemNum];
            for (int cellNum = 0; cellNum < item.Cells.Count; cellNum++)
            {
                worksheet.Cells[itemNum, cellNum].SetValue(GetCellText(item.Cells[cellNum].Text));
            }
        }

        for (int colNum = 0; colNum < gridColumns.Length; colNum++)
        {
            if (!gridColumns[colNum].Visible)
            {
                worksheet.Columns[colNum].SetWidth(new ColumnWidth(0, false));
            }
        }

        GenerateFile(workbook, fileName);
    }

    private void GenerateFile(object structure, string fileName)
    {
        RadFlowDocument flowDoc = structure as RadFlowDocument;
        Workbook workbook = structure as Workbook;

        byte[] output;

        using (MemoryStream ms = new MemoryStream())
        {
            if (flowDoc != null)
            {
                DocxFormatProvider provider = new DocxFormatProvider();
                provider.Export(flowDoc, ms);
            }
            else
            {
                XlsxFormatProvider provider = new XlsxFormatProvider();
                provider.Export(workbook, ms);
            }
            ms.Position = 0;
            output = ms.ToArray();
        }

        Response.ContentType = flowDoc != null ? "application/msword" : "application/excel";
        Response.AddHeader("content-disposition", "attachment; filename=" + fileName);
        Response.BinaryWrite(output);
        Response.Flush();
        Response.Close();
    }

    private string GetCellText(string text)
    {
        return text.Replace("&nbsp;", "");
    }

    private void IgnorePaging()
    {
        if (RadGrid1.ExportSettings.IgnorePaging)
        {
            RadGrid1.AllowPaging = false;
            RadGrid1.Rebind();
        }
    }

    protected void RadGrid1_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        List<dynamic> dataList = new List<dynamic>();
        for (int i = 1; i < 1000; i++)
        {
            dataList.Add(new { Index = i, Name = "User " + i, Age = i / 100 + 22 });
        }

        RadGrid1.DataSource = dataList;
    }
}